use chronoptics_tof as tof;
use tof::Camera;

fn stream_captured_frames(file_name: &str) -> tof::Result<()> {
    let mut cam = tof::CsfCamera::new(&tof::ProcessingConfig::new_default()?, file_name)?;

    // Get ideal default processing for current camera config
    let cam_config = cam.get_camera_config()?;
    let default_processing_config = cam_config.default_processing()?;
    cam.set_process_config(&default_processing_config)?;

    let nr_streams = tof::select_streams(&cam, &[tof::FrameType::Z, tof::FrameType::Xyz])?;
    println!("Selected {nr_streams} streams");

    cam.start()?;

    let nr_frames = cam.get_frame_number()?;

    for _ in 0..nr_frames {
        let mut frames = cam.get_frames()?;

        for frame in &mut frames {
            print!(
                "Frame type is {}",
                tof::frame_type_to_string(frame.frame_type()?)?
            );

            match frame.frame_type()? {
                tof::FrameType::Z => {
                    // Access const data
                    let data = frame.as_slice::<f32>()?;

                    println!(" len is {}", data.len());

                    // Access mut data
                    let _data = frame.as_mut_slice::<f32>()?;
                }
                tof::FrameType::Xyz => {
                    // Access const data
                    let data = frame.as_slice::<tof::XYZ>()?;

                    println!(" len is {}", data.len());

                    // Access mut data
                    let _data = frame.as_mut_slice::<tof::XYZ>()?;
                }
                _ => println!("Unhandled frame type"),
            }
        }
    }

    Ok(())
}

fn main() {
    let file_name = "raw_frames.csf";
    let ret = stream_captured_frames(file_name);

    match ret {
        Ok(_) => println!("Stream captured frames ran succesfully"),
        Err(err) => println!("Failed with {err}"),
    }
}
